from django.db import models
from .base import OCDBase, LinkBase, OCDIDField, RelatedBase, IdentifierBase
from .division import Division
from .jurisdiction import Jurisdiction
from .. import common

# abstract models

class ContactDetailBase(RelatedBase):
    type = models.CharField(max_length=50, choices=common.CONTACT_TYPE_CHOICES)
    value = models.CharField(max_length=300)
    note = models.CharField(max_length=300, blank=True)
    label = models.CharField(max_length=300, blank=True)

    class Meta:
        abstract = True

    def __str__(self):
        return '{}: {}'.format(self.get_type_display(), self.value)


class OtherNameBase(RelatedBase):
    name = models.CharField(max_length=500, db_index=True)
    note = models.CharField(max_length=500, blank=True)
    start_date = models.CharField(max_length=10)    # YYYY[-MM[-DD]]
    end_date = models.CharField(max_length=10)      # YYYY[-MM[-DD]]

    class Meta:
        abstract = True


# the actual models

class Organization(OCDBase):
    id = OCDIDField(ocd_type='organization')
    name = models.CharField(max_length=300)
    image = models.URLField(blank=True, max_length=2000)
    parent = models.ForeignKey('self', related_name='children', null=True)
    jurisdiction = models.ForeignKey(Jurisdiction, related_name='organizations', null=True)
    classification = models.CharField(max_length=100, blank=True,
                                      choices=common.ORGANIZATION_CLASSIFICATION_CHOICES)
    founding_date = models.CharField(max_length=10, blank=True)     # YYYY[-MM[-DD]]
    dissolution_date = models.CharField(max_length=10, blank=True)  # YYYY[-MM[-DD]]

    def __str__(self):
        return self.name

    class Meta:
        index_together = [
            ['jurisdiction', 'classification', 'name'],
            ['classification', 'name'],
        ]


class OrganizationIdentifier(IdentifierBase):
    organization = models.ForeignKey(Organization, related_name='identifiers')

    def __str__(self):
        tmpl = '%s identifies %s'
        return tmpl % (self.identifier, self.organization)


class OrganizationName(OtherNameBase):
    organization = models.ForeignKey(Organization, related_name='other_names')


class OrganizationContactDetail(ContactDetailBase):
    organization = models.ForeignKey(Organization, related_name='contact_details')


class OrganizationLink(LinkBase):
    organization = models.ForeignKey(Organization, related_name='links')


class OrganizationSource(LinkBase):
    organization = models.ForeignKey(Organization, related_name='sources')


class Post(OCDBase):
    id = OCDIDField(ocd_type='post')
    label = models.CharField(max_length=300)
    role = models.CharField(max_length=300, blank=True)
    organization = models.ForeignKey(Organization, related_name='posts')
    division = models.ForeignKey(Division, related_name='posts', null=True, default=None)
    start_date = models.CharField(max_length=10)    # YYYY[-MM[-DD]]
    end_date = models.CharField(max_length=10)    # YYYY[-MM[-DD]]

    class Meta:
        index_together = [
            ['organization', 'label']
        ]


class PostContactDetail(ContactDetailBase):
    post = models.ForeignKey(Post, related_name='contact_details')


class PostLink(LinkBase):
    post = models.ForeignKey(Post, related_name='links')


class Person(OCDBase):
    id = OCDIDField(ocd_type='person')
    name = models.CharField(max_length=300, db_index=True)
    sort_name = models.CharField(max_length=100, default='')

    image = models.URLField(blank=True, max_length=2000)
    gender = models.CharField(max_length=100)
    summary = models.CharField(max_length=500)
    national_identity = models.CharField(max_length=300)
    biography = models.TextField()
    birth_date = models.CharField(max_length=10, blank=True)    # YYYY[-MM[-DD]]
    death_date = models.CharField(max_length=10, blank=True)    # YYYY[-MM[-DD]]

    def __str__(self):
        return self.name

    class Meta:
        verbose_name_plural = "people"


class PersonIdentifier(IdentifierBase):
    person = models.ForeignKey(Person, related_name='identifiers')


class PersonName(OtherNameBase):
    person = models.ForeignKey(Person, related_name='other_names')


class PersonContactDetail(ContactDetailBase):
    person = models.ForeignKey(Person, related_name='contact_details')


class PersonLink(LinkBase):
    person = models.ForeignKey(Person, related_name='links')


class PersonSource(LinkBase):
    person = models.ForeignKey(Person, related_name='sources')


class Membership(OCDBase):
    id = OCDIDField(ocd_type='membership')
    organization = models.ForeignKey(Organization, related_name='memberships')
    person = models.ForeignKey(Person, related_name='memberships')
    post = models.ForeignKey(Post, related_name='memberships', null=True)
    on_behalf_of = models.ForeignKey(Organization, related_name='memberships_on_behalf_of',
                                     null=True)
    label = models.CharField(max_length=300, blank=True)
    role = models.CharField(max_length=300, blank=True)
    start_date = models.CharField(max_length=10, blank=True)    # YYYY[-MM[-DD]]
    end_date = models.CharField(max_length=10, blank=True)      # YYYY[-MM[-DD]]

    class Meta:
        index_together = [
            ['organization', 'person', 'label', 'post']
        ]


class MembershipContactDetail(ContactDetailBase):
    membership = models.ForeignKey(Membership, related_name='contact_details')


class MembershipLink(LinkBase):
    membership = models.ForeignKey(Membership, related_name='links')



from django.contrib import admin
from opencivicdata.models import division as models


@admin.register(models.Division)
class DivisionAdmin(admin.ModelAdmin):
    list_display = ('name', 'id')
    search_fields = list_display




from django.contrib import admin
from django.template import defaultfilters
from opencivicdata.models import bill as models


@admin.register(models.BillAction)
class BillActionAdmin(admin.ModelAdmin):
    readonly_fields = ('id', 'bill', 'organization')
    list_selected_related = (
        'bill',
        'bill__legislative_session',
        'bill__legislative_session__jurisdiction')
    list_display = ('bill', 'date', 'description')


class BillActionInline(admin.TabularInline):
    model = models.BillAction
    readonly_fields = ('date', 'organization', 'description')
    fields = ('date', 'description', 'organization')
    extra = 0


class BillSponsorshipInline(admin.TabularInline):
    model = models.BillSponsorship
    readonly_fields = ('name',)
    fields = ('name', 'primary', 'classification')
    extra = 0


@admin.register(models.Bill)
class BillAdmin(admin.ModelAdmin):
    readonly_fields = ('from_organization', 'legislative_session')
    search_fields = ['identifier', 'title',]
    fields = (
        'identifier', 'legislative_session', 'classification',
        'from_organization', 'title', 'id', 'extras')
    list_selected_related = (
        'sources',
        'legislative_session',
        'legislative_session__jurisdiction')
    inlines = [BillActionInline, BillSponsorshipInline]

    def get_jurisdiction_name(self, obj):
        return obj.legislative_session.jurisdiction.name
    get_jurisdiction_name.short_description = 'Jurisdiction'

    def get_session_name(self, obj):
        return obj.legislative_session.name
    get_session_name.short_description = 'Session'

    def source_link(self, obj):
        source = obj.sources.filter(url__icontains="legislationdetail").get()
        tmpl = u'<a href="{0}" target="_blank">View source</a>'
        return tmpl.format(source.url)
    source_link.short_description = 'View source'
    source_link.allow_tags = True

    def get_truncated_sponsors(self, obj):
        spons = ', '.join(s.name for s in obj.sponsorships.all()[:5])
        return defaultfilters.truncatewords(spons, 10)
    get_truncated_sponsors.short_description = 'Sponsors'

    def get_truncated_title(self, obj):
        return defaultfilters.truncatewords(obj.title, 25)
    get_truncated_title.short_description = 'Title'

    list_display = (
        'identifier', 'get_jurisdiction_name',
        'get_session_name', 'get_truncated_sponsors',
        'get_truncated_title', 'source_link')

    list_filter = ('legislative_session__jurisdiction__name',)


@admin.register(models.BillAbstract)
class BillAbstractAdmin(admin.ModelAdmin):
    pass


@admin.register(models.BillTitle)
class BillTitleAdmin(admin.ModelAdmin):
    pass


@admin.register(models.BillIdentifier)
class BillIdentifierAdmin(admin.ModelAdmin):
    list_display = ('bill', 'identifier')
    readonly_fields = ('bill',)


@admin.register(models.BillActionRelatedEntity)
class BillActionRelatedEntityAdmin(admin.ModelAdmin):
    pass


@admin.register(models.RelatedBill)
class RelatedBillAdmin(admin.ModelAdmin):
    pass


@admin.register(models.BillSponsorship)
class BillSponsorshipAdmin(admin.ModelAdmin):
    raw_id_fields = ('bill', 'person', 'organization')
    list_selected_related = (
        'person',
        'organization',
        'bill_legislative_session',
        'bill_legislative_session__jurisdiction')


@admin.register(models.BillDocument)
class BillDocumentAdmin(admin.ModelAdmin):
    pass


@admin.register(models.BillVersion)
class BillVersionAdmin(admin.ModelAdmin):
    pass


@admin.register(models.BillDocumentLink)
class BillDocumentLinkAdmin(admin.ModelAdmin):
    pass


@admin.register(models.BillVersionLink)
class BillVersionLinkAdmin(admin.ModelAdmin):
    pass


@admin.register(models.BillSource)
class BillSourceAdmin(admin.ModelAdmin):
    list_display = ('bill', 'url', 'note')
    readonly_fields = ('bill',)




import pytest
from opencivicdata.models import (Jurisdiction, LegislativeSession, Division,
                                  Organization, OrganizationIdentifier, OrganizationName,
                                  OrganizationContactDetail, OrganizationSource,
                                  Person, PersonIdentifier, PersonName, PersonContactDetail,
                                  PersonLink, PersonSource, Post, PostContactDetail, PostLink,
                                  Membership, MembershipContactDetail, MembershipLink)
from django.core.exceptions import ValidationError


def test_division_subtypes_from_id():

    # simplest case
    assert Division.subtypes_from_id('ocd-division/country:us') == ({'country': 'us'}, 1)

    # ocd-divison part is optional
    assert Division.subtypes_from_id('country:us/state:ak/county:wild') == (
        {'country': 'us', 'subtype1': 'state', 'subid1': 'ak', 'subtype2': 'county',
         'subid2': 'wild'}, 3)

    # country is not optional
    with pytest.raises(ValueError):
        Division.subtypes_from_id('state:nc/city:raleigh')


@pytest.mark.django_db
def test_division_create():
    division_id = 'ocd-division/country:us/state:ak/county:wild'
    d = Division.objects.create(id=division_id, name='Wild County')
    assert d.country == 'us'
    assert d.subtype1 == 'state'
    assert d.subid1 == 'ak'
    assert d.subtype2 == 'county'
    assert d.subid2 == 'wild'
    assert division_id in str(d)


@pytest.mark.django_db
def test_division_children_of():
    us = Division.objects.create('ocd-division/country:us', name='US')
    ak = Division.objects.create('ocd-division/country:us/state:ak', name='Alaska')
    wild = Division.objects.create('ocd-division/country:us/state:ak/county:wild', name='Wild')
    mild = Division.objects.create('ocd-division/country:us/state:ak/county:mild', name='Mild')
    wild_a = Division.objects.create('ocd-division/country:us/state:ak/county:wild/place:a',
                                     name='A')
    wild_b = Division.objects.create('ocd-division/country:us/state:ak/county:wild/place:b',
                                     name='B')
    school = Division.objects.create('ocd-division/country:us/state:ak/county:wild/school:a',
                                     name='A')
    mild_a = Division.objects.create('ocd-division/country:us/state:ak/county:mild/place:a',
                                     name='A')
    mild_a = Division.objects.create('ocd-division/country:us/state:ak/county:mild/place:a/x:y',
                                     name='A')

    # simplest ==
    assert Division.objects.children_of('ocd-division/country:us')[0].id == ak.id

    # 3 divisions within wild county
    assert (Division.objects.children_of('ocd-division/country:us/state:ak/county:wild').count() ==
            3)

    # only one school in wild county
    assert Division.objects.children_of('ocd-division/country:us/state:ak/county:wild',
                                        subtype='school').count() == 1

    # 6 divisions beneath alaska up to 2 levels
    assert Division.objects.children_of('ocd-division/country:us/state:ak', depth=2).count() == 6

    # 7 divisions beneath alaska up to 3 levels
    assert Division.objects.children_of('ocd-division/country:us/state:ak', depth=3).count() == 7


@pytest.mark.django_db
def test_ocdid_default():
    o = Organization.objects.create(name='test org')
    assert o.id.startswith('ocd-organization/')
    assert o.pk == o.id
    p = Person.objects.create(name='test person')
    assert p.id.startswith('ocd-person/')


@pytest.mark.django_db
def test_ocdid_default_nondup():
    """ ensure that defaults actually vary """
    p1 = Person(name='test person 1')
    p2 = Person(name='test person 2')
    assert p1.id != p2.id


@pytest.mark.django_db
def test_ocdid_validation_jurisdiction():
    # this fails
    with pytest.raises(ValidationError):
        j = Jurisdiction(name='test juris', id='ocd-division/country:us/test:something/else',
                         url='http://example.com')
        j.full_clean(exclude=['division'])

    # this succeeds
    j = Jurisdiction(name='test juris', id='ocd-jurisdiction/country:us/test:something/else',
                     url='http://example.com')
    j.full_clean(exclude=['division'])


@pytest.mark.django_db
def test_ocdid_validation_other():
    # this test should handle everything that isn't a jurisdiction

    # this succeeds
    o = Organization(name='test org')
    o.full_clean(exclude=['parent', 'jurisdiction'])

    # this raises
    with pytest.raises(ValidationError):
        o = Organization(name='this is a test', id='ocd-organization/3')
        o.full_clean(exclude=['parent', 'jurisdiction'])



from django.contrib import admin
from django.template import defaultfilters


class LinkAdminInline(admin.TabularInline):
    fields = ('url', 'note')
    extra = 0


class LinkAdmin(admin.ModelAdmin):
    list_display = ('note', 'url')


class MimetypeLinkInline(admin.TabularInline):
    fields = ('media_type', 'url')
    extra = 0


class MimetypeLinkAdmin(admin.ModelAdmin):
    list_display = ('media',)


class IdentifierInline(admin.TabularInline):
    fields = ('identifier', 'scheme')
    extra = 0

class RelatedEntityInline(admin.TabularInline):
    fields = ('name', 'entity_type', 'organization', 'person')


from __future__ import print_function
import io
import os
import csv
from subprocess import check_call
from optparse import make_option

from django.db import transaction, connection
from django.core.management.base import BaseCommand, CommandError

from opencivicdata.divisions import Division as FileDivision
from ...models import Division


def to_db(fd):
    """ convert a FileDivision to a Division """
    args, _ = Division.subtypes_from_id(fd.id)
    if fd.sameAs:
        args['redirect_id'] = fd.sameAs
    return Division(id=fd.id, name=fd.name, **args)


def load_divisions(country):
    existing_divisions = Division.objects.filter(country=country).count()

    country = FileDivision.get('ocd-division/country:' + country)
    objects = [to_db(country)]

    for child in country.children(levels=100):
        objects.append(to_db(child))

    print(len(objects), 'divisions loaded from CSV and ', existing_divisions, 'already in DB')

    if len(objects) == existing_divisions:
        print('no work to be done!')
    else:
        # delete old ids and add new ones all at once
        with transaction.atomic():
            Division.objects.filter(country=country).delete()
            Division.objects.bulk_create(objects, batch_size=10000)
        print(len(objects), 'divisions created')


class Command(BaseCommand):
    help = 'initialize a pupa database'

    def handle(self, *args, **options):
        for country in args:
            load_divisions(country)



from .bill import (BillAdmin, BillAbstractAdmin, BillTitleAdmin,
    BillIdentifierAdmin, BillActionAdmin, BillActionRelatedEntityAdmin,
    RelatedBillAdmin, BillSponsorshipAdmin, BillDocumentAdmin, BillVersionAdmin,
    BillDocumentLinkAdmin, BillVersionLinkAdmin, BillSourceAdmin,)

from .division import DivisionAdmin

from .event import (EventLocationAdmin, EventAdmin, EventMediaAdmin,
    EventMediaLinkAdmin, EventDocumentAdmin, EventDocumentLinkAdmin,
    EventLinkAdmin, EventSourceAdmin, EventParticipantAdmin, EventAgendaItemAdmin,
    EventRelatedEntityAdmin, EventAgendaMediaAdmin, EventAgendaMediaLinkAdmin,)

from .jurisdiction import (JurisdictionAdmin, LegislativeSessionAdmin,)

from .people_orgs import (OrganizationAdmin, OrganizationIdentifierAdmin,
    OrganizationNameAdmin, OrganizationContactDetailAdmin, OrganizationLinkAdmin,
    OrganizationSourceAdmin, PostContactDetailAdmin, PostLinkAdmin, PersonAdmin,
    PersonIdentifierAdmin, PersonNameAdmin, PersonContactDetailAdmin, PersonLinkAdmin,
    PersonSourceAdmin, MembershipAdmin, MembershipContactDetailAdmin, MembershipLinkAdmin,)

from .vote import (VoteEventAdmin, VoteCountAdmin, PersonVoteAdmin, VoteSourceAdmin,)



from django.contrib import admin
from opencivicdata.models import jurisdiction as models


@admin.register(models.Jurisdiction)
class JurisdictionAdmin(admin.ModelAdmin):
    pass


@admin.register(models.LegislativeSession)
class LegislativeSessionAdmin(admin.ModelAdmin):
    pass




from django.core import urlresolvers
from django.contrib import admin
from opencivicdata.models import people_orgs as models

from opencivicdata.admin.base import (
    IdentifierInline, LinkAdmin
    )


class OrganizationIdentifierInline(IdentifierInline):
    model = models.OrganizationIdentifier


@admin.register(models.Organization)
class OrganizationAdmin(admin.ModelAdmin):
    readonly_fields = ('parent', 'jurisdiction')
    fields = (
        'name', 'jurisdiction', 'id', 'classification',
        'parent', ('founding_date', 'dissolution_date'),
        'image', 'extras')
    list_display = ('name', 'jurisdiction')
    inlines = [
        OrganizationIdentifierInline
        ]


@admin.register(models.OrganizationIdentifier)
class OrganizationIdentifierAdmin(admin.ModelAdmin):
    pass


@admin.register(models.OrganizationName)
class OrganizationNameAdmin(admin.ModelAdmin):
    pass


@admin.register(models.OrganizationContactDetail)
class OrganizationContactDetailAdmin(admin.ModelAdmin):
    pass


@admin.register(models.OrganizationLink)
class OrganizationLinkAdmin(admin.ModelAdmin):
    pass


@admin.register(models.OrganizationSource)
class OrganizationSourceAdmin(LinkAdmin):
    list_display = ('organization', 'note', 'url')


@admin.register(models.PostContactDetail)
class PostContactDetailAdmin(admin.ModelAdmin):
    pass


@admin.register(models.PostLink)
class PostLinkAdmin(admin.ModelAdmin):
    pass


class PersonMembershipInline(admin.TabularInline):
    model = models.Membership
    readonly_fields = ('organization', 'post')
    fields = ('organization', 'post', 'label', 'role')
    extra = 0


class PersonIdentifierInline(IdentifierInline):
    model = models.PersonIdentifier


@admin.register(models.Person)
class PersonAdmin(admin.ModelAdmin):
    search_fields = ['name']
    fields = (
        'name', 'id', 'image',
        ('birth_date', 'death_date'),
        'biography', 'extras',
        ('gender', 'national_identity', 'sort_name', 'summary'))

    list_selected_related = (
        'memberships',
        )
    inlines = [PersonMembershipInline, PersonIdentifierInline]

    def get_memberships(self, obj):
        memberships = obj.memberships.select_related('organization__jurisdiction')
        html = []
        SHOW_N = 5
        for memb in memberships[:SHOW_N]:
            info = (memb._meta.app_label, memb._meta.module_name)
            admin_url = urlresolvers.reverse('admin:%s_%s_change' % info, args=(memb.pk,))
            tmpl = '<a href="%s">%s %s</a>\n'
            html.append(tmpl % (
                admin_url,
                memb.organization.jurisdiction.name,
                memb.organization.name))
        more = len(memberships) - SHOW_N
        if 0 < more:
            html.append('And %d more' % more)
        return '<br/>'.join(html)

    get_memberships.short_description = 'Memberships'
    get_memberships.allow_tags = True

    list_display = ('name', 'id', 'get_memberships')


@admin.register(models.PersonIdentifier)
class PersonIdentifierAdmin(admin.ModelAdmin):
    pass


@admin.register(models.PersonName)
class PersonNameAdmin(admin.ModelAdmin):
    pass


@admin.register(models.PersonContactDetail)
class PersonContactDetailAdmin(admin.ModelAdmin):
    pass


@admin.register(models.PersonLink)
class PersonLinkAdmin(admin.ModelAdmin):
    pass


@admin.register(models.PersonSource)
class PersonSourceAdmin(LinkAdmin):
    list_display = ('person', 'note', 'url')


@admin.register(models.Membership)
class MembershipAdmin(admin.ModelAdmin):
    readonly_fields = ('organization', 'person', 'post', 'on_behalf_of')
    list_display = ('organization', 'person', 'post', 'on_behalf_of',
        'label', 'role', 'start_date', 'end_date',)
    fields = ('organization', 'person', 'role', 'post', 'on_behalf_of',
        'label', ('start_date', 'end_date'), 'extras')


@admin.register(models.MembershipContactDetail)
class MembershipContactDetailAdmin(admin.ModelAdmin):
    pass


@admin.register(models.MembershipLink)
class MembershipLinkAdmin(admin.ModelAdmin):
    pass


